<?php

namespace Drupal\exception_mailer\Logger;

use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Logger\LogMessageParserInterface;
use Drupal\Core\Logger\RfcLoggerTrait;
use Drupal\Core\State\StateInterface;
use Psr\Log\LoggerInterface;
use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\Queue\QueueWorkerManagerInterface;
use Drupal\Core\Queue\SuspendQueueException;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Logger\RfcLogLevel;
use Drupal\exception_mailer\ExceptionMailerExcludeManager;
use Drupal\exception_mailer\Utility\UserRepository;

/**
 * Class ErrorLog.
 *
 * Selects errors corresponding levels specified in the config to send emails.
 */
class ErrorLog implements LoggerInterface {
  use RfcLoggerTrait;
  use DependencySerializationTrait;

  /**
   * The message's placeholders parser.
   *
   * @var \Drupal\Core\Logger\LogMessageParserInterface
   */
  protected $parser;

  /**
   * The queue service.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected $queueFactory;

  /**
   * The queue manager.
   *
   * @var \Drupal\Core\Queue\QueueWorkerManagerInterface
   */
  protected $queueManager;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * The state store.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $excludeManager;

  /**
   * Constructs a new ErrorLog object.
   *
   * @param \Drupal\Core\Logger\LogMessageParserInterface $parser
   *   The parser to use when extracting message variables.
   * @param \Drupal\Core\Queue\QueueFactory $queue_factory
   *   The queue service.
   * @param \Drupal\Core\Queue\QueueWorkerManagerInterface $queue_manager
   *   The queue manager.
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   *   The configuration factory.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state.
   * @param \Drupal\exception_mailer\ExceptionMailerExcludeManager $excludeManager
   *   The exception exclude manager.
   */
  public function __construct(
    LogMessageParserInterface $parser,
    QueueFactory $queue_factory,
    QueueWorkerManagerInterface $queue_manager,
    ConfigFactory $configFactory,
    StateInterface $state,
    ExceptionMailerExcludeManager $excludeManager
  ) {
    $this->parser = $parser;
    $this->queueFactory = $queue_factory;
    $this->queueManager = $queue_manager;
    $this->configFactory = $configFactory;
    $this->state = $state;
    $this->excludeManager = $excludeManager;
  }

  /**
   * {@inheritdoc}
   */
  public function log($level, $message, array $context = []) {
    // Remove any backtraces since they may contain an unserializable variable.
    unset($context['backtrace']);

    // Convert PSR3-style messages to \Drupal\Component\Render\FormattableMarkup
    // style, so they can be translated too in runtime.
    $message_placeholders = $this->parser->parseMessagePlaceholders($message, $context);

    $config = $this->configFactory->get('exception_mailer.settings');
    $levels = $config->get('level_type');

    if (empty($levels)) {
      return;
    }

    $queue = $this->queueFactory->get('manual_exception_email', TRUE);
    $queue_worker = $this->queueManager->createInstance('manual_exception_email');

    $severity = RfcLogLevel::getLevels();
    if (\in_array($level, array_values($levels), FALSE)) {
      $data['exception'] = '';
      $data['message'] = empty($message_placeholders) ? $message : strtr($message, $message_placeholders);
      $data['site'] = $this->configFactory->get('system.site')->get('name');
      $data['timestamp'] = $context['timestamp'];
      $data['user'] = $context['uid'];
      $data['severity'] = $severity[$level];
      $data['severity_level'] = $level;
      $data['type'] = mb_substr($context['channel'], 0, 64);
      $data['link'] = $context['link'];
      $data['location'] = $context['request_uri'];
      $data['referrer'] = $context['referer'];
      $data['hostname'] = mb_substr($context['ip'], 0, 128);

      if ($excludes = $this->excludeManager->getExcludes($data, 'error')) {
        /** @var \Drupal\exception_mailer\ExceptionMailerExcludeInterface  $exclude */
        foreach ($excludes as $exclude) {
          if ($exclude->getEmailBody()) {
            $data['email_body'] = $exclude->getEmailBody();
          }
          foreach ($exclude->getEmailAddresses() as $email) {
            $data['email'] = $email;
            $data['exception_mailer_exclude_id'] = $exclude->id();
            $queue->createItem($data);
          }
        }
      }
      else {
        foreach (UserRepository::getUserEmails($config->get('roles')) as $admin) {
          $data['email'] = $admin;
          $queue->createItem($data);
        }
      }

      while ($item = $queue->claimItem()) {
        try {
          $queue_worker->processItem($item->data);
          $queue->deleteItem($item);
          if (isset($item->data['exception_mailer_exclude_id'])) {
            $this->state->set($item->data['exception_mailer_exclude_id'] . '_last_sent', $item->data['timestamp']);
          }
        }
        catch (SuspendQueueException $e) {
          $queue->releaseItem($item);
          break;
        }
        catch (\Exception $e) {
          watchdog_exception('exception_mailer', $e);
        }
      }
    }
  }

}
